@@ -2,6 +2,13 @@
 '''
 Return data to a postgresql server
 
+.. note::
+    :mod:`returners.postgres_local_cache <salt.returners.postgres_local_cache>`
+    is recommended instead of this module when using PostgreSQL as a
+    :ref:`master job cache <external-master-cache>`. These two modules
+    provide different functionality so you should compare each to see which
+    module best suits your particular needs.
+
 :maintainer:    None
 :maturity:      New
 :depends:       psycopg2
@@ -189,7 +196,7 @@ def returner(ret):
     _close_conn(conn)
 
 
-def save_load(jid, load):
+def save_load(jid, load, minions=None):
     '''
     Save the load to the specified jid id
     '''
