@@ -20,7 +20,7 @@ from salttesting.helpers import ensure_in_syspath
 
 ensure_in_syspath('../../')
 
-import tests.integration
+import integration  # pylint: disable=import-error
 
 # Import Salt libs
 # pylint: disable=import-error,no-name-in-module,redefined-builtin
@@ -29,7 +29,7 @@ from salt.ext.six.moves import range
 from salt.config import minion_config
 # pylint: enable=no-name-in-module,redefined-builtin
 
-from salt.loader import LazyLoader, _module_dirs, grains
+from salt.loader import LazyLoader, _module_dirs, grains, utils, proxy, minion_mods
 
 loader_template = '''
 import os
@@ -57,7 +57,7 @@ class LazyLoaderTest(TestCase):
         self.opts['grains'] = grains(self.opts)
 
         # Setup the module
-        self.module_dir = tempfile.mkdtemp(dir=tests.integration.TMP)
+        self.module_dir = tempfile.mkdtemp(dir=integration.TMP)
         self.module_file = os.path.join(self.module_dir,
                                         '{0}.py'.format(self.module_name))
         with open(self.module_file, 'w') as fh:
@@ -243,15 +243,21 @@ class LazyLoaderReloadingTest(TestCase):
     def setUp(self):
         self.opts = _config = minion_config(None)
         self.opts['grains'] = grains(self.opts)
-        self.tmp_dir = tempfile.mkdtemp(dir=tests.integration.TMP)
+        self.tmp_dir = tempfile.mkdtemp(dir=integration.TMP)
 
         self.count = 0
 
         dirs = _module_dirs(self.opts, 'modules', 'module')
         dirs.append(self.tmp_dir)
+        self.utils = utils(self.opts)
+        self.proxy = proxy(self.opts)
+        self.minion_mods = minion_mods(self.opts)
         self.loader = LazyLoader(dirs,
                                  self.opts,
-                                 tag='module')
+                                 tag='module',
+                                 pack={'__utils__': self.utils,
+                                       '__proxy__': self.proxy,
+                                       '__salt__': self.minion_mods})
 
     def tearDown(self):
         shutil.rmtree(self.tmp_dir)
@@ -368,7 +374,7 @@ class LazyLoaderSubmodReloadingTest(TestCase):
     def setUp(self):
         self.opts = _config = minion_config(None)
         self.opts['grains'] = grains(self.opts)
-        self.tmp_dir = tempfile.mkdtemp(dir=tests.integration.TMP)
+        self.tmp_dir = tempfile.mkdtemp(dir=integration.TMP)
         os.makedirs(self.module_dir)
 
         self.count = 0
@@ -376,9 +382,16 @@ class LazyLoaderSubmodReloadingTest(TestCase):
 
         dirs = _module_dirs(self.opts, 'modules', 'module')
         dirs.append(self.tmp_dir)
+        self.utils = utils(self.opts)
+        self.proxy = proxy(self.opts)
+        self.minion_mods = minion_mods(self.opts)
         self.loader = LazyLoader(dirs,
                                  self.opts,
-                                 tag='module')
+                                 tag='module',
+                                 pack={'__utils__': self.utils,
+                                     '__proxy__': self.proxy,
+                                     '__salt__': self.minion_mods}
+                                 )
 
     def tearDown(self):
         shutil.rmtree(self.tmp_dir)
@@ -489,6 +502,94 @@ class LazyLoaderSubmodReloadingTest(TestCase):
         self.assertNotIn(self.module_key, self.loader)
 
 
+mod_template = '''
+def test():
+    return ({val})
+'''
+
+
+class LazyLoaderModulePackageTest(TestCase):
+    '''
+    Test the loader of salt with changing modules
+    '''
+    module_name = 'loadertestmodpkg'
+    module_key = 'loadertestmodpkg.test'
+
+    def setUp(self):
+        self.opts = _config = minion_config(None)
+        self.opts['grains'] = grains(self.opts)
+        self.tmp_dir = tempfile.mkdtemp(dir=integration.TMP)
+
+        dirs = _module_dirs(self.opts, 'modules', 'module')
+        dirs.append(self.tmp_dir)
+        self.loader = LazyLoader(dirs,
+                                 self.opts,
+                                 tag='module')
+
+    def tearDown(self):
+        shutil.rmtree(self.tmp_dir)
+
+    def update_pyfile(self, pyfile, contents):
+        dirname = os.path.dirname(pyfile)
+        if not os.path.exists(dirname):
+            os.makedirs(dirname)
+        with open(pyfile, 'wb') as fh:
+            fh.write(contents)
+            fh.flush()
+            os.fsync(fh.fileno())  # flush to disk
+
+        # pyc files don't like it when we change the original quickly
+        # since the header bytes only contain the timestamp (granularity of seconds)
+        # TODO: don't write them? Is *much* slower on re-load (~3x)
+        # https://docs.python.org/2/library/sys.html#sys.dont_write_bytecode
+        try:
+            os.unlink(pyfile + 'c')
+        except OSError:
+            pass
+
+    def rm_pyfile(self, pyfile):
+        os.unlink(pyfile)
+        os.unlink(pyfile + 'c')
+
+    def update_module(self, relative_path, contents):
+        self.update_pyfile(os.path.join(self.tmp_dir, relative_path), contents)
+
+    def rm_module(self, relative_path):
+        self.rm_pyfile(os.path.join(self.tmp_dir, relative_path))
+
+    def test_module(self):
+        # ensure it doesn't exist
+        self.assertNotIn('foo', self.loader)
+        self.assertNotIn('foo.test', self.loader)
+        self.update_module('foo.py', mod_template.format(val=1))
+        self.loader.clear()
+        self.assertIn('foo.test', self.loader)
+        self.assertEqual(self.loader['foo.test'](), 1)
+
+    def test_package(self):
+        # ensure it doesn't exist
+        self.assertNotIn('foo', self.loader)
+        self.assertNotIn('foo.test', self.loader)
+        self.update_module('foo/__init__.py', mod_template.format(val=2))
+        self.loader.clear()
+        self.assertIn('foo.test', self.loader)
+        self.assertEqual(self.loader['foo.test'](), 2)
+
+    def test_module_package_collision(self):
+        # ensure it doesn't exist
+        self.assertNotIn('foo', self.loader)
+        self.assertNotIn('foo.test', self.loader)
+        self.update_module('foo.py', mod_template.format(val=3))
+        self.loader.clear()
+        self.assertIn('foo.test', self.loader)
+        self.assertEqual(self.loader['foo.test'](), 3)
+
+        self.update_module('foo/__init__.py', mod_template.format(val=4))
+        self.loader.clear()
+        self.assertIn('foo.test', self.loader)
+        self.assertEqual(self.loader['foo.test'](), 4)
+
+
 deep_init_base = '''
 import top_lib
 import top_lib.mid_lib
@@ -511,7 +612,7 @@ class LazyLoaderDeepSubmodReloadingTest(TestCase):
 
     def setUp(self):
         self.opts = _config = minion_config(None)
-        self.tmp_dir = tempfile.mkdtemp(dir=tests.integration.TMP)
+        self.tmp_dir = tempfile.mkdtemp(dir=integration.TMP)
         os.makedirs(self.module_dir)
 
         self.lib_count = collections.defaultdict(int)  # mapping of path -> count
@@ -532,9 +633,16 @@ class LazyLoaderDeepSubmodReloadingTest(TestCase):
 
         dirs = _module_dirs(self.opts, 'modules', 'module')
         dirs.append(self.tmp_dir)
+        self.utils = utils(self.opts)
+        self.proxy = proxy(self.opts)
+        self.minion_mods = minion_mods(self.opts)
         self.loader = LazyLoader(dirs,
                                  self.opts,
-                                 tag='module')
+                                 tag='module',
+                                 pack={'__utils__': self.utils,
+                                       '__proxy__': self.proxy,
+                                       '__salt__': self.minion_mods}
+                                 )
 
     @property
     def module_dir(self):
