@@ -146,31 +146,71 @@ class CPModuleTest(integration.ModuleCase):
 
     def test_get_url(self):
         '''
-        cp.get_url with salt:// source
+        cp.get_url with salt:// source given
         '''
         tgt = os.path.join(integration.TMP, 'scene33')
         self.run_function(
-                'cp.get_url',
-                [
-                    'salt://grail/scene33',
-                    tgt,
-                ])
+            'cp.get_url',
+            [
+                'salt://grail/scene33',
+                tgt,
+            ])
         with salt.utils.fopen(tgt, 'r') as scene:
             data = scene.read()
             self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
             self.assertNotIn('bacon', data)
 
+    def test_get_url_dest_empty(self):
+        '''
+        cp.get_url with salt:// source given and destination omitted.
+        '''
+        ret = self.run_function(
+            'cp.get_url',
+            [
+                'salt://grail/scene33',
+            ])
+        with salt.utils.fopen(ret, 'r') as scene:
+            data = scene.read()
+            self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
+            self.assertNotIn('bacon', data)
+
+    def test_get_url_no_dest(self):
+        '''
+        cp.get_url with salt:// source given and destination set as None
+        '''
+        tgt = None
+        ret = self.run_function(
+            'cp.get_url',
+            [
+                'salt://grail/scene33',
+                tgt,
+            ])
+        self.assertIn('KNIGHT:  They\'re nervous, sire.', ret)
+
+    def test_get_url_nonexistent_source(self):
+        '''
+        cp.get_url with nonexistent salt:// source given
+        '''
+        tgt = None
+        ret = self.run_function(
+            'cp.get_url',
+            [
+                'salt://grail/nonexistent_scene',
+                tgt,
+            ])
+        self.assertEqual(ret, False)
+
     def test_get_url_https(self):
         '''
-        cp.get_url with https:// source
+        cp.get_url with https:// source given
         '''
         tgt = os.path.join(integration.TMP, 'test_get_url_https')
         self.run_function(
-                'cp.get_url',
-                [
-                    'https://repo.saltstack.com/index.html',
-                    tgt,
-                ])
+            'cp.get_url',
+            [
+                'https://repo.saltstack.com/index.html',
+                tgt,
+            ])
         with salt.utils.fopen(tgt, 'r') as instructions:
             data = instructions.read()
             self.assertIn('Bootstrap', data)
@@ -178,6 +218,70 @@ class CPModuleTest(integration.ModuleCase):
             self.assertIn('Windows', data)
             self.assertNotIn('AYBABTU', data)
 
+    def test_get_url_https_dest_empty(self):
+        '''
+        cp.get_url with https:// source given and destination omitted.
+        '''
+        ret = self.run_function(
+            'cp.get_url',
+            [
+                'https://repo.saltstack.com/index.html',
+            ])
+        with salt.utils.fopen(ret, 'r') as instructions:
+            data = instructions.read()
+            self.assertIn('Bootstrap', data)
+            self.assertIn('Debian', data)
+            self.assertIn('Windows', data)
+            self.assertNotIn('AYBABTU', data)
+
+    def test_get_url_https_no_dest(self):
+        '''
+        cp.get_url with https:// source given and destination set as None
+        '''
+        tgt = None
+        ret = self.run_function(
+            'cp.get_url',
+            [
+                'https://repo.saltstack.com/index.html',
+                tgt,
+            ])
+        self.assertIn('Bootstrap', ret)
+        self.assertIn('Debian', ret)
+        self.assertIn('Windows', ret)
+        self.assertNotIn('AYBABTU', ret)
+
+    def test_get_url_file(self):
+        '''
+        cp.get_url with file:// source given
+        '''
+        tgt = ''
+        src = os.path.join('file://', integration.FILES, 'file/base/file.big')
+        ret = self.run_function(
+            'cp.get_url',
+            [
+                src,
+                tgt,
+            ])
+        with salt.utils.fopen(ret, 'r') as scene:
+            data = scene.read()
+            self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
+            self.assertNotIn('bacon', data)
+
+    def test_get_url_file_no_dest(self):
+        '''
+        cp.get_url with file:// source given and destination set as None
+        '''
+        tgt = None
+        src = os.path.join('file://', integration.FILES, 'file/base/file.big')
+        ret = self.run_function(
+            'cp.get_url',
+            [
+                src,
+                tgt,
+            ])
+        self.assertIn('KNIGHT:  They\'re nervous, sire.', ret)
+        self.assertNotIn('bacon', ret)
+
     def test_cache_file(self):
         '''
         cp.cache_file
