@@ -6,6 +6,8 @@
 # Import Python libs
 from __future__ import absolute_import
 from distutils.version import LooseVersion  # pylint: disable=import-error,no-name-in-module
+import pkg_resources
+from pkg_resources import DistributionNotFound
 import random
 import string
 
@@ -22,6 +24,7 @@ import salt.loader
 from salt.modules import boto_vpc
 from salt.exceptions import SaltInvocationError, CommandExecutionError
 from salt.modules.boto_vpc import _maybe_set_name_tag, _maybe_set_tags
+from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
 
 # Import 3rd-party libs
 import salt.ext.six as six
@@ -92,6 +95,19 @@ def _has_required_boto():
         return True
 
 
+def _get_moto_version():
+    '''
+    Returns the moto version
+    '''
+    try:
+        return LooseVersion(moto.__version__)
+    except AttributeError:
+        try:
+            return LooseVersion(pkg_resources.get_distribution('moto').version)
+        except DistributionNotFound:
+            return False
+
+
 def _has_required_moto():
     '''
     Returns True/False boolean depending on if Moto is installed and correct
@@ -100,23 +116,21 @@ def _has_required_moto():
     if not HAS_MOTO:
         return False
     else:
-        try:
-            if LooseVersion(moto.__version__) < LooseVersion(required_moto_version):
-                return False
-        except AttributeError:
-            import pkg_resources
-            from pkg_resources import DistributionNotFound
-            try:
-                if LooseVersion(pkg_resources.get_distribution('moto').version) < LooseVersion(required_moto_version):
-                    return False
-            except DistributionNotFound:
-                return False
+        if _get_moto_version() < LooseVersion(required_moto_version):
+            return False
         return True
 
 
 context = {}
 
 
+@skipIf(NO_MOCK, NO_MOCK_REASON)
+@skipIf(HAS_BOTO is False, 'The boto module must be installed.')
+@skipIf(HAS_MOTO is False, 'The moto module must be installed.')
+@skipIf(_has_required_boto() is False, 'The boto module must be greater than'
+                                       ' or equal to version {0}'
+        .format(required_boto_version))
+@skipIf(_has_required_moto() is False, 'The moto version must be >= to version {0}'.format(required_moto_version))
 class BotoVpcTestCaseBase(TestCase):
     def setUp(self):
         boto_vpc.__context__ = {}
@@ -242,13 +256,6 @@ class BotoVpcTestCaseMixin(object):
         return rtbl
 
 
-@skipIf(NO_MOCK, NO_MOCK_REASON)
-@skipIf(HAS_BOTO is False, 'The boto module must be installed.')
-@skipIf(HAS_MOTO is False, 'The moto module must be installed.')
-@skipIf(_has_required_boto() is False, 'The boto module must be greater than'
-                                       ' or equal to version {0}'
-        .format(required_boto_version))
-@skipIf(_has_required_moto() is False, 'The moto version must be >= to version {0}'.format(required_moto_version))
 class BotoVpcTestCase(BotoVpcTestCaseBase, BotoVpcTestCaseMixin):
     '''
     TestCase for salt.modules.boto_vpc module
@@ -506,13 +513,19 @@ class BotoVpcTestCase(BotoVpcTestCaseBase, BotoVpcTestCaseMixin):
         '''
         Tests describing parameters via vpc id if vpc exist
         '''
+        # With moto 0.4.25 is_default is set to True. 0.4.24 and older, is_default is False
+        if _get_moto_version() >= LooseVersion('0.4.25'):
+            is_default = True
+        else:
+            is_default = False
+
         vpc = self._create_vpc(name='test', tags={'test': 'testvalue'})
 
         describe_vpc = boto_vpc.describe(vpc_id=vpc.id, **conn_parameters)
 
         vpc_properties = dict(id=vpc.id,
                               cidr_block=six.text_type(cidr_block),
-                              is_default=False,
+                              is_default=is_default,
                               state=u'available',
                               tags={u'Name': u'test', u'test': u'testvalue'},
                               dhcp_options_id=u'dopt-7a8b9c2d',
