@@ -9,7 +9,7 @@ import textwrap
 # Import Salt Testing libs
 from salttesting import TestCase
 from salttesting.helpers import ensure_in_syspath
-from salttesting.mock import MagicMock
+from salttesting.mock import MagicMock, patch
 
 ensure_in_syspath('../../')
 
@@ -31,6 +31,7 @@ filemod.__opts__ = {
     'pillar_roots': {'base': 'tmp'},
     'cachedir': 'tmp',
 }
+filemod.__grains__ = {'kernel': 'Linux'}
 
 SED_CONTENT = """test
 some
@@ -39,7 +40,6 @@ content
 here
 """
 
-filemod.__grains__ = {}
 filemod.__pillar__ = {}
 
 
@@ -257,7 +257,7 @@ class FileBlockReplaceTestCase(TestCase):
         )
         with salt.utils.fopen(self.tfile.name, 'r') as fp:
             self.assertNotIn('#-- START BLOCK 2'
-                             + "\n" + new_content + "\n"
+                             + "\n" + new_content
                              + '#-- END BLOCK 2', fp.read())
 
         filemod.blockreplace(self.tfile.name,
@@ -270,7 +270,7 @@ class FileBlockReplaceTestCase(TestCase):
         with salt.utils.fopen(self.tfile.name, 'r') as fp:
             self.assertIn('#-- START BLOCK 2'
                           + "\n" + new_content
-                          + "\n" + '#-- END BLOCK 2', fp.read())
+                          + '#-- END BLOCK 2', fp.read())
 
     def test_replace_append_newline_at_eof(self):
         '''
@@ -284,7 +284,7 @@ class FileBlockReplaceTestCase(TestCase):
                 'content': 'baz',
                 'append_if_not_found': True,
         }
-        block = '{marker_start}\n{content}\n{marker_end}\n'.format(**args)
+        block = '{marker_start}\n{content}{marker_end}\n'.format(**args)
         expected = base + '\n' + block
         # File ending with a newline
         with tempfile.NamedTemporaryFile(mode='w+') as tfile:
@@ -322,7 +322,7 @@ class FileBlockReplaceTestCase(TestCase):
         with salt.utils.fopen(self.tfile.name, 'r') as fp:
             self.assertNotIn(
                 '#-- START BLOCK 2' + "\n"
-                + new_content + "\n" + '#-- END BLOCK 2',
+                + new_content + '#-- END BLOCK 2',
                 fp.read())
 
         filemod.blockreplace(self.tfile.name,
@@ -336,7 +336,7 @@ class FileBlockReplaceTestCase(TestCase):
                 fp.read().startswith(
                     '#-- START BLOCK 2'
                     + "\n" + new_content
-                    + "\n" + '#-- END BLOCK 2'))
+                    + '#-- END BLOCK 2'))
 
     def test_replace_partial_marked_lines(self):
         filemod.blockreplace(self.tfile.name,
@@ -524,6 +524,39 @@ class FileModuleTestCase(TestCase):
         ret = filemod.group_to_gid(group)
         self.assertEqual(ret, group)
 
+    @patch('os.path.isdir', return_value=False)
+    @patch('salt.utils.which', return_value='/bin/patch')
+    def test_patch(self, mock_which, mock_isdir):
+        cmd_mock = MagicMock(return_value='test_retval')
+        with patch.dict(filemod.__salt__, {'cmd.run_all': cmd_mock}):
+            ret = filemod.patch('/path/to/file', '/path/to/patch')
+        cmd = ['/bin/patch', '--forward', '--reject-file=-',
+            '-i', '/path/to/patch', '/path/to/file']
+        cmd_mock.assert_called_once_with(cmd, python_shell=False)
+        self.assertEqual('test_retval', ret)
+
+    @patch('os.path.isdir', return_value=False)
+    @patch('salt.utils.which', return_value='/bin/patch')
+    def test_patch_dry_run(self, mock_which, mock_isdir):
+        cmd_mock = MagicMock(return_value='test_retval')
+        with patch.dict(filemod.__salt__, {'cmd.run_all': cmd_mock}):
+            ret = filemod.patch('/path/to/file', '/path/to/patch', dry_run=True)
+        cmd = ['/bin/patch', '--dry-run', '--forward', '--reject-file=-',
+            '-i', '/path/to/patch', '/path/to/file']
+        cmd_mock.assert_called_once_with(cmd, python_shell=False)
+        self.assertEqual('test_retval', ret)
+
+    @patch('os.path.isdir', return_value=True)
+    @patch('salt.utils.which', return_value='/bin/patch')
+    def test_patch_dir(self, mock_which, mock_isdir):
+        cmd_mock = MagicMock(return_value='test_retval')
+        with patch.dict(filemod.__salt__, {'cmd.run_all': cmd_mock}):
+            ret = filemod.patch('/path/to/dir', '/path/to/patch')
+        cmd = ['/bin/patch', '--forward', '--reject-file=-',
+            '-i', '/path/to/patch', '-d', '/path/to/dir', '--strip=0']
+        cmd_mock.assert_called_once_with(cmd, python_shell=False)
+        self.assertEqual('test_retval', ret)
+
     def test_apply_template_on_contents(self):
         '''
         Tests that the templating engine works on string contents
