@@ -40,7 +40,7 @@ class MountTestCase(TestCase):
         '''
         List the active mounts.
         '''
-        with patch.dict(mount.__grains__, {'os': 'FreeBSD'}):
+        with patch.dict(mount.__grains__, {'os': 'FreeBSD', 'kernel': 'FreeBSD'}):
             # uid=user1 tests the improbable case where a OS returns a name
             # instead of a numeric id, for #25293
             mock = MagicMock(return_value='A B C D,E,F,uid=user1,gid=grp1')
@@ -56,7 +56,7 @@ class MountTestCase(TestCase):
                                                             'gid=100'],
                                                    'fstype': 'C'}})
 
-        with patch.dict(mount.__grains__, {'os': 'Solaris'}):
+        with patch.dict(mount.__grains__, {'os': 'Solaris', 'kernel': 'SunOS'}):
             mock = MagicMock(return_value='A * B * C D/E/F')
             with patch.dict(mount.__salt__, {'cmd.run_stdout': mock}):
                 self.assertEqual(mount.active(), {'B':
@@ -64,17 +64,17 @@ class MountTestCase(TestCase):
                                                    'opts': ['D', 'E', 'F'],
                                                    'fstype': 'C'}})
 
-        with patch.dict(mount.__grains__, {'os': 'OpenBSD'}):
+        with patch.dict(mount.__grains__, {'os': 'OpenBSD', 'kernel': 'OpenBSD'}):
             mock = MagicMock(return_value={})
             with patch.object(mount, '_active_mounts_openbsd', mock):
                 self.assertEqual(mount.active(), {})
 
-        with patch.dict(mount.__grains__, {'os': 'MacOS'}):
+        with patch.dict(mount.__grains__, {'os': 'MacOS', 'kernel': 'Darwin'}):
             mock = MagicMock(return_value={})
             with patch.object(mount, '_active_mounts_darwin', mock):
                 self.assertEqual(mount.active(), {})
 
-        with patch.dict(mount.__grains__, {'os': 'MacOS'}):
+        with patch.dict(mount.__grains__, {'os': 'MacOS', 'kernel': 'Darwin'}):
             mock = MagicMock(return_value={})
             with patch.object(mount, '_active_mountinfo', mock):
                 with patch.object(mount, '_active_mounts_darwin', mock):
@@ -89,35 +89,62 @@ class MountTestCase(TestCase):
             self.assertEqual(mount.fstab(), {})
 
         mock = MagicMock(return_value=True)
+        with patch.dict(mount.__grains__, {'kernel': ''}):
+            with patch.object(os.path, 'isfile', mock):
+                file_data = '\n'.join(['#',
+                                       'A B C D,E,F G H'])
+                with patch('salt.utils.fopen',
+                           mock_open(read_data=file_data),
+                           create=True) as m:
+                    m.return_value.__iter__.return_value = file_data.splitlines()
+                    self.assertEqual(mount.fstab(), {'B': {'device': 'A',
+                                                           'dump': 'G',
+                                                           'fstype': 'C',
+                                                           'opts': ['D', 'E', 'F'],
+                                                           'pass': 'H'}})
+
+    def test_vfstab(self):
+        '''
+        List the content of the vfstab
+        '''
+        mock = MagicMock(return_value=False)
         with patch.object(os.path, 'isfile', mock):
-            file_data = '\n'.join(['#',
-                                   'A B C D,E,F G H'])
-            with patch('salt.utils.fopen',
-                       mock_open(read_data=file_data),
-                       create=True) as m:
-                m.return_value.__iter__.return_value = file_data.splitlines()
-                self.assertEqual(mount.fstab(), {'B': {'device': 'A',
-                                                       'dump': 'G',
-                                                       'fstype': 'C',
-                                                       'opts': ['D', 'E', 'F'],
-                                                       'pass': 'H'}})
+            self.assertEqual(mount.vfstab(), {})
+
+        mock = MagicMock(return_value=True)
+        with patch.dict(mount.__grains__, {'kernel': 'SunOS'}):
+            with patch.object(os.path, 'isfile', mock):
+                file_data = '\n'.join(['#',
+                                       'swap        -   /tmp                tmpfs    -   yes    size=2048m'])
+                with patch('salt.utils.fopen',
+                           mock_open(read_data=file_data),
+                           create=True) as m:
+                    m.return_value.__iter__.return_value = file_data.splitlines()
+                    self.assertEqual(mount.fstab(), {'/tmp': {'device': 'swap',
+                                                              'device_fsck': '-',
+                                                              'fstype': 'tmpfs',
+                                                              'mount_at_boot': 'yes',
+                                                              'opts': ['size=2048m'],
+                                                              'pass_fsck': '-'}})
 
     def test_rm_fstab(self):
         '''
         Remove the mount point from the fstab
         '''
         mock_fstab = MagicMock(return_value={})
-        with patch.object(mount, 'fstab', mock_fstab):
-            with patch('salt.utils.fopen', mock_open()):
-                self.assertTrue(mount.rm_fstab('name', 'device'))
+        with patch.dict(mount.__grains__, {'kernel': ''}):
+            with patch.object(mount, 'fstab', mock_fstab):
+                with patch('salt.utils.fopen', mock_open()):
+                    self.assertTrue(mount.rm_fstab('name', 'device'))
 
         mock_fstab = MagicMock(return_value={'name': 'name'})
-        with patch.object(mount, 'fstab', mock_fstab):
-            with patch('salt.utils.fopen', mock_open()) as m_open:
-                helper_open = m_open()
-                helper_open.write.assertRaises(CommandExecutionError,
-                                               mount.rm_fstab,
-                                               config=None)
+        with patch.dict(mount.__grains__, {'kernel': ''}):
+            with patch.object(mount, 'fstab', mock_fstab):
+                with patch('salt.utils.fopen', mock_open()) as m_open:
+                    m_open.side_effect = IOError(13, 'Permission denied:', '/file')
+                    self.assertRaises(CommandExecutionError,
+                                      mount.rm_fstab,
+                                      'name', 'device')
 
     def test_set_fstab(self):
         '''
@@ -153,11 +180,7 @@ class MountTestCase(TestCase):
 
         mock = MagicMock(return_value={'name': 'name'})
         with patch.object(mount, 'fstab', mock):
-            with patch('salt.utils.fopen', mock_open()) as m_open:
-                helper_open = m_open()
-                helper_open.write.assertRaises(CommandExecutionError,
-                                               mount.rm_automaster,
-                                               'name', 'device')
+            self.assertTrue(mount.rm_automaster('name', 'device'))
 
     def test_set_automaster(self):
         '''
@@ -248,7 +271,7 @@ class MountTestCase(TestCase):
 
         file_data = '\n'.join(['Filename Type Size Used Priority',
                                '/dev/sda1 partition 31249404 4100 -1'])
-        with patch.dict(mount.__grains__, {'os': ''}):
+        with patch.dict(mount.__grains__, {'os': '', 'kernel': ''}):
             with patch('salt.utils.fopen',
                        mock_open(read_data=file_data),
                        create=True) as m:
@@ -263,7 +286,7 @@ class MountTestCase(TestCase):
         file_data = '\n'.join(['Device Size Used Unknown Unknown Priority',
                                '/dev/sda1 31249404 4100 unknown unknown -1'])
         mock = MagicMock(return_value=file_data)
-        with patch.dict(mount.__grains__, {'os': 'OpenBSD'}):
+        with patch.dict(mount.__grains__, {'os': 'OpenBSD', 'kernel': 'OpenBSD'}):
             with patch.dict(mount.__salt__, {'cmd.run_stdout': mock}):
                 self.assertDictEqual(mount.swaps(), {'/dev/sda1':
                                                      {'priority': '-1',
@@ -276,44 +299,50 @@ class MountTestCase(TestCase):
         Activate a swap disk
         '''
         mock = MagicMock(return_value={'name': 'name'})
-        with patch.object(mount, 'swaps', mock):
-            self.assertEqual(mount.swapon('name'),
-                             {'stats': 'name', 'new': False})
+        with patch.dict(mount.__grains__, {'kernel': ''}):
+            with patch.object(mount, 'swaps', mock):
+                self.assertEqual(mount.swapon('name'),
+                                 {'stats': 'name', 'new': False})
 
         mock = MagicMock(return_value={})
-        with patch.object(mount, 'swaps', mock):
-            mock = MagicMock(return_value=None)
-            with patch.dict(mount.__salt__, {'cmd.run': mock}):
-                self.assertEqual(mount.swapon('name', False), {})
+        with patch.dict(mount.__grains__, {'kernel': ''}):
+            with patch.object(mount, 'swaps', mock):
+                mock = MagicMock(return_value=None)
+                with patch.dict(mount.__salt__, {'cmd.run': mock}):
+                    self.assertEqual(mount.swapon('name', False), {})
 
         mock = MagicMock(side_effect=[{}, {'name': 'name'}])
-        with patch.object(mount, 'swaps', mock):
-            mock = MagicMock(return_value=None)
-            with patch.dict(mount.__salt__, {'cmd.run': mock}):
-                self.assertEqual(mount.swapon('name'), {'stats': 'name',
-                                                        'new': True})
+        with patch.dict(mount.__grains__, {'kernel': ''}):
+            with patch.object(mount, 'swaps', mock):
+                mock = MagicMock(return_value=None)
+                with patch.dict(mount.__salt__, {'cmd.run': mock}):
+                    self.assertEqual(mount.swapon('name'), {'stats': 'name',
+                                                            'new': True})
 
     def test_swapoff(self):
         '''
         Deactivate a named swap mount
         '''
         mock = MagicMock(return_value={})
-        with patch.object(mount, 'swaps', mock):
-            self.assertEqual(mount.swapoff('name'), None)
+        with patch.dict(mount.__grains__, {'kernel': ''}):
+            with patch.object(mount, 'swaps', mock):
+                self.assertEqual(mount.swapoff('name'), None)
 
         mock = MagicMock(return_value={'name': 'name'})
-        with patch.object(mount, 'swaps', mock):
-            with patch.dict(mount.__grains__, {'os': 'test'}):
-                mock = MagicMock(return_value=None)
-                with patch.dict(mount.__salt__, {'cmd.run': mock}):
-                    self.assertFalse(mount.swapoff('name'))
+        with patch.dict(mount.__grains__, {'kernel': ''}):
+            with patch.object(mount, 'swaps', mock):
+                with patch.dict(mount.__grains__, {'os': 'test'}):
+                    mock = MagicMock(return_value=None)
+                    with patch.dict(mount.__salt__, {'cmd.run': mock}):
+                        self.assertFalse(mount.swapoff('name'))
 
         mock = MagicMock(side_effect=[{'name': 'name'}, {}])
-        with patch.object(mount, 'swaps', mock):
-            with patch.dict(mount.__grains__, {'os': 'test'}):
-                mock = MagicMock(return_value=None)
-                with patch.dict(mount.__salt__, {'cmd.run': mock}):
-                    self.assertTrue(mount.swapoff('name'))
+        with patch.dict(mount.__grains__, {'kernel': ''}):
+            with patch.object(mount, 'swaps', mock):
+                with patch.dict(mount.__grains__, {'os': 'test'}):
+                    mock = MagicMock(return_value=None)
+                    with patch.dict(mount.__salt__, {'cmd.run': mock}):
+                        self.assertTrue(mount.swapoff('name'))
 
     def test_is_mounted(self):
         '''
