@@ -969,6 +969,7 @@ class FileTestCase(TestCase):
     # 'comment' function tests: 1
 
     @destructiveTest
+    @patch.object(os.path, 'exists', MagicMock(return_value=True))
     def test_comment(self):
         '''
         Test to comment out specified lines in a file.
@@ -1027,6 +1028,7 @@ class FileTestCase(TestCase):
     # 'uncomment' function tests: 1
 
     @destructiveTest
+    @patch.object(os.path, 'exists', MagicMock(return_value=True))
     def test_uncomment(self):
         '''
         Test to uncomment specified commented lines in a file
@@ -1079,88 +1081,6 @@ class FileTestCase(TestCase):
                         ret.update({'comment': comt, 'result': True})
                         self.assertDictEqual(filestate.uncomment(name, regex), ret)
 
-    # 'append' function tests: 1
-
-    def test_append(self):
-        '''
-        Test to ensure that some text appears at the end of a file.
-        '''
-        name = '/etc/motd'
-        source = ['salt://motd/hr-messages.tmpl']
-        sources = ['salt://motd/devops-messages.tmpl']
-        text = ['Trust no one unless you have eaten much salt with him.']
-
-        ret = {'name': name,
-               'result': False,
-               'comment': '',
-               'pchanges': {},
-               'changes': {}}
-
-        comt = ('Must provide name to file.append')
-        ret.update({'comment': comt, 'name': ''})
-        self.assertDictEqual(filestate.append(''), ret)
-
-        comt = ('source and sources are mutually exclusive')
-        ret.update({'comment': comt, 'name': name})
-        self.assertDictEqual(filestate.append(name, source=source,
-                                              sources=sources), ret)
-
-        mock_t = MagicMock(return_value=True)
-        mock_f = MagicMock(return_value=False)
-        mock_err = MagicMock(side_effect=[TypeError, True, True])
-        with patch.dict(filestate.__salt__,
-                        {'file.directory_exists': mock_f,
-                         'file.makedirs': mock_t,
-                         'file.stats': mock_f,
-                         'cp.get_template': mock_f,
-                         'file.contains_regex_multiline': mock_err,
-                         'file.search': mock_err}):
-            with patch.object(os.path, 'isdir', mock_t):
-                comt = ('The following files will be changed:\n/etc:'
-                        ' directory - new\n')
-                ret.update({'comment': comt, 'name': name, 'pchanges': {'/etc': {'directory': 'new'}}})
-                self.assertDictEqual(filestate.append(name, makedirs=True), ret)
-
-            with patch.object(os.path, 'isabs', mock_f):
-                comt = ('Specified file {0} is not an absolute path'
-                        .format(name))
-                ret.update({'comment': comt, 'pchanges': {}})
-                self.assertDictEqual(filestate.append(name), ret)
-
-            with patch.object(os.path, 'isabs', mock_t):
-                with patch.object(os.path, 'exists', mock_t):
-                    comt = ("Failed to load template file {0}".format(source))
-                    ret.pop('pchanges')
-                    ret.update({'comment': comt, 'name': source, 'data': [], })
-                    self.assertDictEqual(filestate.append(name, source=source),
-                                         ret)
-
-                    ret.pop('data', None)
-                    ret.update({'name': name})
-                    with patch.object(salt.utils, 'fopen',
-                                      MagicMock(mock_open(read_data=''))):
-                        comt = ('No text found to append. Nothing appended')
-                        ret.update({'comment': comt, 'pchanges': {}})
-                        self.assertDictEqual(filestate.append(name, text=text),
-                                             ret)
-
-                        with patch.object(salt.utils, 'istextfile', mock_f):
-                            with patch.dict(filestate.__opts__, {'test': True}):
-                                change = {'diff': 'Replace binary file'}
-                                ret.update({'comment': '', 'result': None,
-                                            'changes': change})
-                                self.assertDictEqual(filestate.append
-                                                     (name, text=text), ret)
-
-                            with patch.dict(filestate.__opts__,
-                                            {'test': False}):
-                                comt = ('File {0} is in correct state'
-                                        .format(name))
-                                ret.update({'comment': comt, 'result': True,
-                                            'changes': {}})
-                                self.assertDictEqual(filestate.append
-                                                     (name, text=text), ret)
-
     # 'prepend' function tests: 1
 
     def test_prepend(self):
@@ -1449,26 +1369,6 @@ class FileTestCase(TestCase):
                                                      (name, source,
                                                       preserve=True), ret)
 
-                    with patch.object(os.path, 'isdir', mock_t):
-                        with patch.dict(filestate.__opts__, {'test': False}):
-                            with patch.object(shutil, 'copy',
-                                              MagicMock(side_effect=[IOError,
-                                                                     True])):
-                                comt = ('Failed to copy "{0}" to "{1}"'
-                                        .format(source, name))
-                                ret.update({'comment': comt, 'result': False})
-                                self.assertDictEqual(filestate.copy
-                                                     (name, source,
-                                                      preserve=True), ret)
-
-                                comt = ('Copied "{0}" to "{1}"'.format(source,
-                                                                       name))
-                                ret.update({'comment': comt, 'result': True,
-                                            'changes': {name: source}})
-                                self.assertDictEqual(filestate.copy
-                                                     (name, source,
-                                                      preserve=True), ret)
-
     # 'rename' function tests: 1
 
     def test_rename(self):
